

#if !defined __VDIS_H
#define __VDIS_H


#include "vissim.h"
#include "dismailbox.h"
#include "discoord.h"


#define DIS_FLAG_NULL   0x00
#define DIS_FLAG_UPDATE 0x01


/* **************************************************************
************************************************************** */
class dis_em_systems_type : public dbl_llist {

   public:
      int id;
      DIS_emissions_beams_type *es_radar;

      virtual ~dis_em_systems_type() {}
};


/* **************************************************************
************************************************************** */
class dis_data : public platform_data {

   protected:
      DISPDU_entity_state_type es;
      DISPDU_emissions_type em;
      dbl_llist_manager sensorman;

      unsigned int flags;
      double next_update_time;
      double update_time;
      buffer_type send_buffer;
   
   public:
      void label(dbl_llist_manager *label_manager);
      void update(double *machine_time);
      int  perform(iface *isim, int task, void *data);
      void render(iface *isim);
      int  init(iface *isim, correlate *basis, symbol_correlation_type *str);

      dis_data(basic_platform *x);
      virtual ~dis_data();
};


class dis_iface;

/* **************************************************************
************************************************************** */
class dis_platform : public basic_platform {

   protected:
      // basic_platform
      void update_data(double *machine_time);

   public:
      // dis_platform
      DIS_id_type dis_id;
      
      virtual ~dis_platform() { }

      int init(dis_iface *isim, DISPDU_entity_state_type *es, symbol_correlation_type *str);
      void update(dis_iface *isim, DISPDU_entity_state_type *es, double *machine_time);
};


/* **************************************************************
************************************************************** */
class dis_mcp_platform : public dis_platform {

   protected:
      DISPDU_entity_state_type es;
      buffer_type send_buffer;
      double update_time, next_update_time;
      
   public:
      // basic_platform
      void update_platform(plat_stat_type *pc);

      // dis_mcp_platform
      dis_mcp_platform();
      virtual ~dis_mcp_platform() { }

      int init(dis_iface *isim, symbol_correlation_type *str);
};


/* **************************************************************
   This class is the base class for any simulation manager
************************************************************** */
class dis_iface : public iface {

   protected:
      asockudp clientudp;
      int exercise_id;
      int site_id;
      int app_id;
      lla_type center;
      dis_mcp_platform control_platform;
      int cockpit_code;
      
      dis_platform *AddPlatform(DISPDU_entity_state_type *es, double *machine_time);

   public:

      // iface
      void init();
      void cleanup();
      void parse_input(int argc, char **argv);
      void update(double *machine_time);
      void mcp(plat_stat_type *pc);

      // dis_iface
      int id_counter;

      dis_iface();
      virtual ~dis_iface() {}
};


extern dispost_office_type disiface;

extern char TOKEN_APP_STR[];
extern char TOKEN_SITE_STR[];
extern char TOKEN_EXERCISE_STR[];

extern char TOKEN_COCKPIT_APP_STR[];
extern char TOKEN_COCKPIT_SITE_STR[];
extern char TOKEN_COCKPIT_ID_STR[];

#endif
