

#include <string.h>
#include "vgev_icon.h"


icon_manager_type iconman;


/* **************************************************************
************************************************************** */
int icon_manager_type::read_data(char *filename) {

   sfile sinfile;
   char token[MAXSTRLEN];
   icon_type *ptr;

   iconman.dest();

   if (!sinfile.scan_data(filename, "", PLATFORM_SLASH))
      return 0;

   while (!sinfile.seof() && sinfile.scan_token(token)) {
      iconman.append(ptr = new icon_type, NULL);
      ptr->scan_data(&sinfile, token);
   }

   return 1;
}


/* **************************************************************
************************************************************** */
void icon_type::render(canvas *tgev, unsigned char *color, int x, int y) {

   vector_base_type *ptr;

   if (tgev && color)
      tgev->tpColor3ubv(color, 1);

   for (ptr = (vector_base_type *)drawman.head; ptr; ptr = (vector_base_type *)ptr->next)
      ptr->render(x,y);
}


/* **************************************************************
************************************************************** */
void icon_type::scan_data(sfile *sinfile, char *token) {

   vector_base_type *ptr;

   type = atoi(token);

   sinfile->scan_token(token); // '{'

   while (sinfile->scan_token(token) && strcmp(token, "}")) {

      if (!strcmp(token, "point")) {
         drawman.append(ptr = new vector_point_type, NULL);
         ptr->scan_data(sinfile, token);
      }

      else if (!strcmp(token, "line")) {
         drawman.append(ptr = new vector_line_type, NULL);
         ptr->scan_data(sinfile, token);
      }

      else if (!strcmp(token, "arc")) {
         drawman.append(ptr = new vector_arc_type, NULL);
         ((vector_arc_type *)ptr)->fill_flag = GL_LINE_LOOP;
         ptr->scan_data(sinfile, token);
      }

      else if (!strcmp(token, "fill_arc")) {
         drawman.append(ptr = new vector_arc_type, NULL);
         ((vector_arc_type *)ptr)->fill_flag = GL_POLYGON;
         ptr->scan_data(sinfile, token);
      }

      else if (!strcmp(token, "polygon")) {
         drawman.append(ptr = new vector_polygon_type, NULL);
         ((vector_polygon_type *)ptr)->fill_flag = GL_LINE_LOOP;
         ptr->scan_data(sinfile, token);
      }

      else if (!strcmp(token, "fill_polygon")) {
         drawman.append(ptr = new vector_polygon_type, NULL);
         ((vector_polygon_type *)ptr)->fill_flag = GL_POLYGON;
         ptr->scan_data(sinfile, token);
      }

   }

}


/* **************************************************************
************************************************************** */
vector_point_type::vector_point_type() {

   pt = NULL;
}


/* **************************************************************
************************************************************** */
vector_point_type::~vector_point_type() {

   if (pt)
      delete [] pt;
}


/* **************************************************************
************************************************************** */
void vector_point_type::scan_data(sfile *sinfile, char *token) {

   int i;

   sinfile->scan_token(token);

   count = atoi(token);

   if (pt)
      delete [] pt;

   pt = new vector2d[count];

   for (i=0; i<count; i++) {
      sinfile->scan_token(token);
      pt[i][0] = atof(token);

      sinfile->scan_token(token);
      pt[i][1] = atof(token);
   }

}


/* **************************************************************
************************************************************** */
void vector_point_type::render(int x, int y) {

   int i;

   if (!pt)
      return;

   glBegin(GL_POINTS);
      for (i=0; i<count; i++)
         glVertex2dv(pt[i]);
   glEnd();
}

      
/* **************************************************************
************************************************************** */
vector_arc_type::vector_arc_type() {

   fill_flag = GL_LINE_LOOP;
}


/* **************************************************************
************************************************************** */
void vector_arc_type::scan_data(sfile *sinfile, char *token) {

   int i;
   double radius, start, end;
   double t, dt;

   sinfile->scan_token(token);
   radius = atof(token);

   sinfile->scan_token(token);
   start = atof(token);

   sinfile->scan_token(token);
   end = atof(token);

   t = deg2rad(start);
   dt = deg2rad((end - start)/31.0);

   for (i=0; i<31; i++, t+=dt) {
      pt[i][0] = cos(t)*radius;
      pt[i][1] = sin(t)*radius;
   }

   t = deg2rad(end);
   pt[i][0] = cos(t)*radius;
   pt[i][1] = sin(t)*radius;
}


/* **************************************************************
************************************************************** */
void vector_arc_type::render(int x, int y) {

   int i;

   glBegin(fill_flag);
      for (i=0; i<32; i++)
         glVertex2d(pt[i][0]+x, pt[i][1]+y);
   glEnd();
}

      
/* **************************************************************
************************************************************** */
vector_polygon_type::vector_polygon_type() {

   pt = NULL;
}

      
/* **************************************************************
************************************************************** */
vector_polygon_type::~vector_polygon_type() {

   if (pt)
      delete [] pt;
}


/* **************************************************************
************************************************************** */
void vector_polygon_type::scan_data(sfile *sinfile, char *token) {

   int i;

   sinfile->scan_token(token);
   count = atoi(token);

   if (pt)
      delete [] pt;

   pt = new vector2d[count];

   for (i=0; i<count; i++) {
      sinfile->scan_token(token);
      pt[i][0] = atof(token);

      sinfile->scan_token(token);
      pt[i][1] = atof(token);
   }

}


/* **************************************************************
************************************************************** */
void vector_polygon_type::render(int x, int y) {

   int i;

   glBegin(fill_flag);
      for (i=0; i<count; i++)
         glVertex2d(pt[i][0]+x, pt[i][1]+y);
   glEnd();
}

      
/* **************************************************************
************************************************************** */
vector_line_type::vector_line_type() {

   pt = NULL;
}

      
/* **************************************************************
************************************************************** */
vector_line_type::~vector_line_type() {

   if (pt)
      delete [] pt;
}


/* **************************************************************
************************************************************** */
void vector_line_type::scan_data(sfile *sinfile, char *token) {

   int i;

   sinfile->scan_token(token);
   count = atoi(token);

   if (pt)
      delete [] pt;

   pt = new vector4d[count];

   for (i=0; i<count; i++) {
      sinfile->scan_token(token);
      pt[i][0] = atof(token);

      sinfile->scan_token(token);
      pt[i][1] = atof(token);

      sinfile->scan_token(token);
      pt[i][2] = atof(token);

      sinfile->scan_token(token);
      pt[i][3] = atof(token);
   }

}


/* **************************************************************
************************************************************** */
void vector_line_type::render(int x, int y) {

   int i;

   glBegin(GL_LINES);

      for (i=0; i<count; i++) {
         glVertex2d(pt[i][0]+x, pt[i][1]+y);
         glVertex2d(pt[i][2]+x, pt[i][3]+y);
      }

   glEnd();
}

